/*

  This file is part of sniffer, a packet capture utility and
  network moniter
  The author can be contacted at <mistral@stev.org>
  the lastest version is avilable from 
  http://stev.org

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

	$Id: sniff.c,v 1.2 2011/12/30 13:53:47 james.stevenson Exp $

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <pthread.h>
#include <time.h>
#include <fcntl.h>

#include <signal.h>

#include <sys/types.h>
#include <sys/ioctl.h>
#include <sys/socket.h>

#include <linux/if_packet.h>

#include "config.h"
#include "locks.h"
#include "tidy.h"
#include "gui_main.h"
#include "log.h"
#include "lookup.h"
#include "arp.h"
#include "ip.h"
#include "tcp.h"
#include "udp.h"
#include "icmp.h"
#include "if.h"
#include "sniff.h"



#define SOCK_THREADS 2

/* structs for threads and mutex */
pthread_attr_t thread_attr;
pthread_t sock_thread[SOCK_THREADS];
pthread_t gui_thread;

int sockfd; /* our main socket for sniffing */

int main(int argc, char **argv) {
	int i, cury = 1;

	sockfd = socket(PF_PACKET, SOCK_RAW, htons(ETH_P_ALL) );
	if (sockfd < 0) {
		fprintf(stderr, "You must be ROOT to run this\n");
		perror("socket");
		exit(1);
	}

	/* set up the threads so that they automatically join */
	if (pthread_attr_init(&thread_attr) != 0) {
		fprintf(stderr, "pthread Attrb\n");
		exit(1);
	}

	if (pthread_attr_setdetachstate(&thread_attr, PTHREAD_CREATE_DETACHED) != 0) {
		fprintf(stderr, "pthread Attrb\n");
		exit(1);
	}

	if (pthread_create(&gui_thread, &thread_attr, gui_main, NULL) != 0 ) {
		fprintf(stderr, "pthread Attrb\n");
		exit(1);
	}

	/* we need to wait until the gui is set up before we do anything else */
	while(!gui_ready) { 
	}

	/* this must be started after the gui in case of errors */
	tidy_init();
	log_start();	/* start the logging */
	mvwprintw(gui->twin, cury++, 1, "Starting logs\r");
	wrefresh(gui->twin);

	init_stat(&stat_global, "ALL");
	init_stat(&ip_stat, "IP");
	init_stat(&tcp_stat, "TCP");
	init_stat(&udp_stat, "UDP");
	init_stat(&icmp_stat,"ICMP");
	mvwprintw(gui->twin, cury++, 1, "Starting Stats\r");
	wrefresh(gui->twin);

#ifdef ARP
	arp_init(); /* fire up the arp */
	mvwprintw(gui->twin, cury++, 1, "Starting ARP\r");
	wrefresh(gui->twin);
#endif

	if_init();
	mvwprintw(gui->twin, cury++, 1, "Starting Interface\r");
	wrefresh(gui->twin);

	if (ip_init() >= 0) {
		mvwprintw(gui->twin, cury++, 1, "Starting IPv4 With: ");
		wrefresh(gui->twin);
		if (tcp_init() >= 0) {
			wprintw(gui->twin, "TCP ");
			wrefresh(gui->twin);
		}
		if (udp_init() >= 0) {
			wprintw(gui->twin, "UDP ");
			wrefresh(gui->twin);
		}
		if (icmp_init() >= 0) {
			wprintw(gui->twin, "ICMP ");
			wrefresh(gui->twin);
		}
	} /* end of ip_init */


	if (pthread_create(&lookup_thread, &thread_attr, lookup_init, NULL) != 0 ) {
		log_errno("thread lookup_thread");
		exit(1);
	}

	while(lookup_active == 0) { }

	mvwprintw(gui->twin, cury++, 1, "Starting Lookup Thread\r");
	wrefresh(gui->twin);


	for(i=0;i<SOCK_THREADS;i++) {
		if (pthread_create(&sock_thread[i], NULL, sock_read, NULL) != 0 ) {
			log_errno("thread sock_thread");
			exit(1);
		} else {
			mvwprintw(gui->twin, cury++, 1, "Starting Thread %d\r", i);
			wrefresh(gui->twin);
		}
	}

	gui_ready = 2;
	tidy();

	return 0;
}

/* The new way to read from the socket */
/* this should be able to tell that interface the packet came from */
/* and then send it to the correct handler */
void *sock_read(void *arg) {
	struct sniff_pkt pkt;
	int fromlen = sizeof(pkt.fromaddr); /* need this because of the way recvfrom is done */

	pkt.data = malloc(8192);
	if (!pkt.data) {
		log_errno("malloc");
		pthread_exit(0);
	}

	/* we need to get the first packet */
	pkt.readlen = recvfrom(sockfd, pkt.data, 8191, 0, (struct sockaddr *) &pkt.fromaddr, &fromlen );

	while (pkt.readlen > 0) {
		pkt.dataleft = pkt.readlen;
		pkt.cur = pkt.data;
		pkt.face = NULL;
		
		stat_global.packets++;
		stat_global.bytes += pkt.readlen;

		if_handle(&pkt);

		/* get the next packet */
		/* it appreas we dont need locking here */
		pkt.readlen = recvfrom(sockfd, pkt.data, 8191, 0, (struct sockaddr *) &pkt.fromaddr, &fromlen );
	}
	/* if we come out of the loop we got a read error */
	log_errno("read");
	log_error("Thread Exiting %d\n", getpid());
	pthread_exit(0);
	return NULL;
}
